@@ -163,10 +163,11 @@ define([
 
       var allCellsButton = $('<button/>')
       .append($('<i/>').addClass('fa fa-arrows-v'))
+      .attr('id', 'findreplace_allcells_btn')
       .attr('type', 'button')
       .addClass("btn btn-default btn-sm")
       .attr('data-toggle','button')
-      .attr('title', i18n.msg._('Replace in selected cells'));
+      .attr('title', i18n.msg._('Replace in all cells'));
 
     var isCaseSensitiveButton = $('<button/>')
       .attr('type', 'button')
@@ -179,6 +180,7 @@ define([
      
     var search  = $("<input/>")
       .addClass('form-control input-sm')
+      .attr('id', 'findreplace_find_inp')
       .attr('placeholder',i18n.msg._('Find'));
 
     var findFormGroup = $('<div/>').addClass('form-group');
@@ -194,6 +196,7 @@ define([
     )
 
     var replace = $("<input/>")
+      .attr('id', 'findreplace_replace_inp')
       .addClass('form-control input-sm')
       .attr('placeholder',i18n.msg._('Replace'));
     var replaceFormGroup = $('<div/>').addClass('form-group');
@@ -354,7 +357,8 @@ define([
       keyboard_manager: env.notebook.keyboard_manager,
       buttons:{
         'Replace All':{ class: "btn-primary",
-            click: function(event){onsubmit(event); return true;}
+            click: function(event){onsubmit(event); return true;},
+            id: "findreplace_replaceall_btn",
         }
       },
       open: function(){
